@@ -8,11 +8,17 @@
 #include "helpers.h"
 #include "utf.h"
 
-typedef enum {comment_hash, comment_xml, comment_c, comment_not_found} comment_style;
+typedef enum {
+    comment_hash,
+    comment_xml,
+    comment_c,
+    comment_js,
+    comment_not_found
+} COMMENT_STYLE;
 
 typedef struct {
     const char *extension;
-    comment_style comment;
+    COMMENT_STYLE comment;
 } SCRIPT_FORMAT;
 
 const SCRIPT_FORMAT supported_formats[] = {
@@ -23,21 +29,24 @@ const SCRIPT_FORMAT supported_formats[] = {
     {".psm1",   comment_hash},
     {".cdxml",  comment_xml},
     {".mof",    comment_c},
+    {".js",     comment_js},
     {NULL,      comment_not_found},
 };
 
-const char *signature_header = "SIG # Begin signature block";
-const char *signature_footer = "SIG # End signature block";
+#define header_hash "SIG # Begin signature block"
+#define footer_hash "SIG # End signature block"
+#define header_js "Begin signature block"
+#define footer_js "End signature block"
 
 typedef struct {
-    const char *open;
-    const char *close;
+    const char *open, *close, *header, *footer;
 } SCRIPT_COMMENT;
 
 const SCRIPT_COMMENT comment_text[] = {
-    [comment_hash] = {"# ", ""},
-    [comment_xml]  = {"<!-- ", " -->"},
-    [comment_c]    = {"/* ", " */"}
+    [comment_hash] = {"# ", "", header_hash, footer_hash},
+    [comment_xml]  = {"<!-- ", " -->", header_hash, footer_hash},
+    [comment_c]    = {"/* ", " */", header_hash, footer_hash},
+    [comment_js]   = {"// SIG // ", "", header_js, footer_js}
 };
 
 struct script_ctx_st {
@@ -242,11 +251,18 @@ static u_char *script_digest_calc(FILE_FORMAT_CTX *ctx, const EVP_MD *md)
     u_char *mdbuf;
     BIO *hash = BIO_new(BIO_f_md());
 
+#if defined(__GNUC__)
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wcast-qual"
+#endif
     if (!BIO_set_md(hash, md)) {
-        printf("Unable to set the message digest of BIO\n");
+        fprintf(stderr, "Unable to set the message digest of BIO\n");
         BIO_free_all(hash);
         return NULL; /* FAILED */
     }
+#if defined(__GNUC__)
+#pragma GCC diagnostic pop
+#endif
     BIO_push(hash, BIO_new(BIO_s_null()));
     if (!script_write_bio(hash, ctx->options->indata, ctx->script_ctx->fileend)) {
         BIO_free_all(hash);
@@ -286,7 +302,7 @@ static int script_verify_digests(FILE_FORMAT_CTX *ctx, PKCS7 *p7)
         }
     }
     if (mdtype == -1) {
-        printf("Failed to extract current message digest\n\n");
+        fprintf(stderr, "Failed to extract current message digest\n\n");
         return 0; /* FAILED */
     }
     md = EVP_get_digestbynid(mdtype);
@@ -299,7 +315,7 @@ static int script_verify_digests(FILE_FORMAT_CTX *ctx, PKCS7 *p7)
     BIO_free_all(bhash);
 
     if (!compare_digests(mdbuf, cmdbuf, mdtype)) {
-        printf("Signature verification: failed\n\n");
+        fprintf(stderr, "Signature verification: failed\n\n");
         OPENSSL_free(cmdbuf);
         return 0; /* FAILED */
     }
@@ -316,20 +332,18 @@ static PKCS7 *script_pkcs7_extract(FILE_FORMAT_CTX *ctx)
 {
     const char *signature_data = ctx->options->indata + ctx->script_ctx->sigpos;
     size_t signature_len = ctx->script_ctx->fileend - ctx->script_ctx->sigpos;
-    size_t base64_len, der_max_length, der_length;
+    size_t base64_len;
     char *ptr;
     BIO *bio_mem, *bio_b64 = NULL;
     char *base64_data = NULL;
-    char *der_data = NULL;
-    const char *der_tmp;
     char *clean_base64 = NULL;
     int clean_base64_len = 0;
     const char *open_tag = ctx->script_ctx->comment_text->open;
     const char *close_tag = ctx->script_ctx->comment_text->close;
     size_t open_tag_len = strlen(open_tag);
     size_t close_tag_len = strlen(close_tag);
-    size_t signature_header_len = strlen(signature_header);
-    size_t signature_footer_len = strlen(signature_footer);
+    size_t header_len = strlen(ctx->script_ctx->comment_text->header);
+    size_t footer_len = strlen(ctx->script_ctx->comment_text->footer);
     PKCS7 *retval = NULL;
 
     if (!script_check_file(ctx)) {
@@ -339,6 +353,8 @@ static PKCS7 *script_pkcs7_extract(FILE_FORMAT_CTX *ctx)
     if (ctx->script_ctx->utf == 8) {
         base64_len = signature_len;
         base64_data = OPENSSL_malloc(base64_len);
+        if (!base64_data)
+            return NULL; /* memory allocation failed */
         memcpy(base64_data, signature_data, base64_len);
     } else {
         base64_len = utf16_to_utf8((const void *)signature_data,
@@ -348,7 +364,7 @@ static PKCS7 *script_pkcs7_extract(FILE_FORMAT_CTX *ctx)
     /* allocate memory for cleaned Base64 */
     clean_base64 = OPENSSL_malloc(base64_len);
     if (!clean_base64) {
-        printf("Malloc failed\n");
+        fprintf(stderr, "Malloc failed\n");
         goto cleanup;
     }
 
@@ -357,7 +373,7 @@ static PKCS7 *script_pkcs7_extract(FILE_FORMAT_CTX *ctx)
         /* find the opening tag */
         for(;;) {
             if (ptr + open_tag_len >= base64_data + base64_len) {
-                printf("Signature line too long\n");
+                fprintf(stderr, "Signature line too long\n");
                 goto cleanup;
             }
             if (!memcmp(ptr, open_tag, (size_t)open_tag_len)) {
@@ -366,18 +382,18 @@ static PKCS7 *script_pkcs7_extract(FILE_FORMAT_CTX *ctx)
             }
             ptr++;
         }
-        /* process signature_header and signature_footer */
-        if (ptr + signature_header_len < base64_data + base64_len &&
-                !memcmp(ptr, signature_header, signature_header_len))
-            ptr += signature_header_len;
-        if (ptr + signature_footer_len <= base64_data + base64_len &&
-                !memcmp(ptr, signature_footer, signature_footer_len))
+        /* process header and footer */
+        if (ptr + header_len < base64_data + base64_len &&
+                !memcmp(ptr, ctx->script_ctx->comment_text->header, header_len))
+            ptr += header_len;
+        if (ptr + footer_len <= base64_data + base64_len &&
+                !memcmp(ptr, ctx->script_ctx->comment_text->footer, footer_len))
             break; /* success */
 
         /* copy until the closing tag */
         for(;;) {
             if (ptr + close_tag_len >= base64_data + base64_len) {
-                printf("Signature line too long\n");
+                fprintf(stderr, "Signature line too long\n");
                 goto cleanup;
             }
             if (close_tag_len) {
@@ -403,26 +419,12 @@ static PKCS7 *script_pkcs7_extract(FILE_FORMAT_CTX *ctx)
     BIO_push(bio_b64, bio_mem);
     BIO_set_flags(bio_b64, BIO_FLAGS_BASE64_NO_NL);
 
-    /* allocate memory for DER output */
-    der_max_length = BIO_ctrl_pending(bio_b64);
-    der_data = OPENSSL_malloc(der_max_length);
-    if (!der_data)
-        goto cleanup;
-
-    /* decode Base64 to DER */
-    if (!BIO_read_ex(bio_b64, der_data, der_max_length, &der_length))
-        goto cleanup;
-    if (der_length <= 0)
-        goto cleanup;
-
     /* decode DER */
-    der_tmp = der_data;
-    retval = d2i_PKCS7(NULL, (const unsigned char **)&der_tmp, (int)der_length);
+    retval = d2i_PKCS7_bio(bio_b64, NULL);
 
 cleanup:
     OPENSSL_free(base64_data);
     OPENSSL_free(clean_base64);
-    OPENSSL_free(der_data);
     BIO_free_all(bio_b64);
     return retval;
 }
@@ -471,10 +473,10 @@ static int script_process_data(FILE_FORMAT_CTX *ctx, BIO *hash, BIO *outdata)
         ctx->script_ctx->fileend = ctx->script_ctx->sigpos;
     }
     if (!script_write_bio(outdata, ctx->options->indata, ctx->script_ctx->fileend))
-        return 1; /* FAILED */
+        return 0; /* FAILED */
     if (!script_digest_convert(hash, ctx, ctx->script_ctx->fileend))
-        return 1; /* FAILED */
-    return 0; /* OK */
+        return 0; /* FAILED */
+    return 1; /* OK */
 }
 
 /*
@@ -489,21 +491,21 @@ static PKCS7 *script_pkcs7_signature_new(FILE_FORMAT_CTX *ctx, BIO *hash)
     PKCS7 *p7 = pkcs7_create(ctx);
 
     if (!p7) {
-        printf("Creating a new signature failed\n");
+        fprintf(stderr, "Creating a new signature failed\n");
         return NULL; /* FAILED */
     }
     if (!add_indirect_data_object(p7)) {
-        printf("Adding SPC_INDIRECT_DATA_OBJID failed\n");
+        fprintf(stderr, "Adding SPC_INDIRECT_DATA_OBJID failed\n");
         PKCS7_free(p7);
         return NULL; /* FAILED */
     }
     content = spc_indirect_data_content_get(hash, ctx);
     if (!content) {
-        printf("Failed to get spcIndirectDataContent\n");
+        fprintf(stderr, "Failed to get spcIndirectDataContent\n");
         return NULL; /* FAILED */
     }
     if (!sign_spc_indirect_data_content(p7, content)) {
-        printf("Failed to set signed content\n");
+        fprintf(stderr, "Failed to set signed content\n");
         PKCS7_free(p7);
         ASN1_OCTET_STRING_free(content);
         return NULL; /* FAILED */
@@ -547,7 +549,9 @@ static int script_append_pkcs7(FILE_FORMAT_CTX *ctx, BIO *outdata, PKCS7 *p7)
     (void)BIO_set_close(bio, BIO_NOCLOSE);
 
     /* split to individual lines and write to outdata */
-    if (!write_commented(ctx, outdata, signature_header, strlen(signature_header)))
+    if (!write_commented(ctx, outdata,
+            ctx->script_ctx->comment_text->header,
+            strlen(ctx->script_ctx->comment_text->header)))
         goto cleanup;
     for (i = 0; i < buffer->length; i += 64) {
         if (!write_commented(ctx, outdata, buffer->data + i,
@@ -555,7 +559,9 @@ static int script_append_pkcs7(FILE_FORMAT_CTX *ctx, BIO *outdata, PKCS7 *p7)
             goto cleanup;
         }
     }
-    if (!write_commented(ctx, outdata, signature_footer, strlen(signature_footer)))
+    if (!write_commented(ctx, outdata,
+            ctx->script_ctx->comment_text->footer,
+            strlen(ctx->script_ctx->comment_text->footer)))
         goto cleanup;
 
     /* signtool expects CRLF terminator at the end of the text file */
@@ -621,7 +627,7 @@ static SCRIPT_CTX *script_ctx_get(char *indata, uint32_t filesize, const SCRIPT_
             *ptr && commented_header_len < commented_header_size;
             commented_header_len++)
         ptr = utf8DecodeRune(ptr, 1, commented_header + commented_header_len);
-    for (ptr = signature_header;
+    for (ptr = comment->header;
             *ptr && commented_header_len < commented_header_size;
             commented_header_len++)
         ptr = utf8DecodeRune(ptr, 1, commented_header + commented_header_len);
@@ -687,6 +693,8 @@ static int write_commented(FILE_FORMAT_CTX *ctx, BIO *outdata, const char *data,
      * - closing tag
      * - trailing NUL ("\0") */
     line = OPENSSL_malloc(2 + open_tag_len + length + close_tag_len + 1);
+    if (!line)
+        return 0; /* memory allocation failed */
     strcpy(line, "\r\n");
     strcat(line, open_tag);
     memcpy(line + 2 + open_tag_len, data, length);
@@ -798,15 +806,21 @@ static BIO *script_digest_calc_bio(FILE_FORMAT_CTX *ctx, const EVP_MD *md)
         fileend = ctx->script_ctx->sigpos;
     else
         fileend = ctx->script_ctx->fileend;
-
+#if defined(__GNUC__)
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wcast-qual"
+#endif
     if (!BIO_set_md(hash, md)) {
-        printf("Unable to set the message digest of BIO\n");
+        fprintf(stderr, "Unable to set the message digest of BIO\n");
         BIO_free_all(hash);
         return NULL; /* FAILED */
     }
+#if defined(__GNUC__)
+#pragma GCC diagnostic pop
+#endif
     BIO_push(hash, BIO_new(BIO_s_null()));
     if (!script_digest_convert(hash, ctx, fileend)) {
-        printf("Unable calc a message digest value\n");
+        fprintf(stderr, "Unable calc a message digest value\n");
         BIO_free_all(hash);
         return NULL; /* FAILED */
     }
@@ -868,12 +882,12 @@ static int script_write_bio(BIO *bio, char *indata, size_t len)
 static int script_check_file(FILE_FORMAT_CTX *ctx)
 {
     if (!ctx) {
-        printf("Init error\n\n");
+        fprintf(stderr, "Init error\n");
         return 0; /* FAILED */
     }
     if (ctx->script_ctx->sigpos == 0
         || ctx->script_ctx->sigpos > ctx->script_ctx->fileend) {
-        printf("No signature found\n\n");
+        fprintf(stderr, "No signature found\n");
         return 0; /* FAILED */
     }
 
